package cz.zcu.kiv.cosi.parkoviste.parkoviste;

import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import cz.zcu.kiv.cosi.parkoviste.IKonfigurace;
import cz.zcu.kiv.cosi.parkoviste.IPrijezdyOdjezdy;

public class Parkoviste implements IPrijezdyOdjezdy, ApplicationContextAware {
	
	private ApplicationContext context;
	
	private static final int MISTA_AUTO = 1;
	private static final int MISTA_BUS = 2;
	private static final int MISTA_TIR = 4;
	
	private int kapacita;
	private int volnaMista;
	
	private int pocetAuta;
	private int pocetBus;
	private int pocetTir;
	
	public Parkoviste(IKonfigurace konfigurace) {
		this.kapacita = this.volnaMista = konfigurace.getKapacita();
		this.pocetAuta = 0;
		this.pocetBus = 0;
		this.pocetTir = 0;		
	}
	
	@Override
	public void setApplicationContext(ApplicationContext applicationContext)
			throws BeansException {
		this.context = applicationContext;		
	}

	private void posliZpravuMista(){
		context.publishEvent(new ParkovisteZbyvaEvent(this, volnaMista));
		
		if(volnaMista == 0){
			posliZpravuPlno();
		}
	}
	
	private void posliZpravuPlno(){
		context.publishEvent(new ParkovistePlneEvent(this));
	}
	
	@Override
	public void odjelBus() throws Exception {
		if(((volnaMista + MISTA_BUS) <= kapacita) && pocetBus > 0){
			volnaMista += MISTA_BUS;
			
			pocetBus--;
			posliZpravuMista();
		}else{
			throw new Exception("Neni bus k odjezdu!");
		}
	}

	@Override
	public void odjelTir() throws Exception {
		if(((volnaMista + MISTA_TIR) <= kapacita) && pocetTir > 0){
			volnaMista += MISTA_TIR;
			
			pocetTir--;
			posliZpravuMista();
		}else{
			throw new Exception("Neni tir k odjezdu!");
		}
	}

	@Override
	public void odjeloAuto() throws Exception {
		if(((volnaMista + MISTA_AUTO) <= kapacita) && pocetAuta > 0){
			volnaMista += MISTA_AUTO;
			
			pocetAuta--;
			posliZpravuMista();
		}else{
			throw new Exception("Neni auto k odjezdu!");
		}
	}

	@Override
	public void prijelBus() throws Exception {
		if((volnaMista - MISTA_BUS) >= 0){
			volnaMista -= MISTA_BUS;
			
			pocetBus++;
			posliZpravuMista();
		}else{
			
			if(volnaMista == 0){
				posliZpravuMista();
			}
			
			throw new Exception("Nedostatek mist pro bus!");
		}
	}

	@Override
	public void prijelTir() throws Exception {
		if((volnaMista - MISTA_TIR) >= 0){
			volnaMista -= MISTA_TIR;
			
			pocetTir++;
			posliZpravuMista();
		}else{
			
			if(volnaMista == 0){
				posliZpravuMista();
			}
			
			throw new Exception("Nedostatek mist pro tir!");
		}
	}

	@Override
	public void prijeloAuto() throws Exception{
		if((volnaMista - MISTA_AUTO) >= 0){
			volnaMista -= MISTA_AUTO;
			
			pocetAuta++;
			posliZpravuMista();
		}else{
			
			posliZpravuMista();
			throw new Exception("Nedostatek mist pro auto!");
		}
	}

}
